////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                    Addon Functions                     //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Create a new addon ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char AddonCreate(struct WORLD *world, unsigned short type)
{
	struct ADDON *addon;

	if (world->addonCount < ADDON_COUNT)
	{
		addon = &world->addonList[world->addonCount++];

		addon->time = 0;
		addon->type = type;

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Destroy a bonus ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char AddonDestroy(struct WORLD *world, unsigned short index)
{
	if (index < world->addonCount)
	{
		memcpy(&world->addonList[index],&world->addonList[world->addonCount - 1],sizeof(*world->addonList));
		world->addonCount--;

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Update Addons ^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void AddonUpdate(struct WORLD *world)
{
	unsigned char alive;
	unsigned short i;
	struct ADDON *addon;

//--------------- Update all active addons ---------------//

	for (i = world->addonCount; i--;)
	{
		addon = &world->addonList[i];
		alive = 1;

//---------------------- Draw addon ----------------------//

		switch (addon->type)
		{
			case ADDON_DEFLECTOR:
				if ((!world->lost) && (addon->time & 1))
					GrayClipSprite32_TRANW_R((world->shipList[0].x >> COEF_SCREEN_TO_ZONE) - 8,(world->shipList[0].y >> COEF_SCREEN_TO_ZONE) - 8,32,(unsigned long *)(gfxAddon),(unsigned long *)(gfxAddon) + 32,world->light,world->dark);
				else if (addon->time > 15)
					alive = 0;
			break;
		}

		addon->time++;

//-------------------- Destroy addon ---------------------//

		if (!alive)
		{
			AddonDestroy(world,i);
		}
	}
}
