////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                    Bonus Functions                     //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Create a new bonus ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char BonusCreate(struct WORLD *world, unsigned short type, unsigned short x, unsigned short y)
{
	struct BONUS *bonus;

	if (world->bonusCount < BONUS_COUNT)
	{
		bonus = &world->bonusList[world->bonusCount++];

		bonus->sprite = type;
		bonus->value = bonusValue[type];
		bonus->x = x;
		bonus->y = y;

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Destroy a bonus ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char BonusDestroy(struct WORLD *world, unsigned short index)
{
	if (index < world->bonusCount)
	{
		memcpy(&world->bonusList[index],&world->bonusList[world->bonusCount - 1],sizeof(*world->bonusList));
		world->bonusCount--;

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Update Bonuses ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void BonusUpdate(struct WORLD *world)
{
	unsigned short i;
	struct BONUS *bonus;
	struct SHIP *ship;

//-------------- Update all active bonuses ---------------//

	for (i = world->bonusCount; i--;)
	{
		bonus = &world->bonusList[i];

//---------------------- Move bonus ----------------------//

		if (!(world->lost) && (world->addonLevel[ADDON_MAGNETIC] != 255))
		{
			ship = &world->shipList[0];

			if (bonus->x < ship->x - ADDON_MAGNETIC_DISTANCE)
				bonus->x += (world->addonLevel[ADDON_MAGNETIC] + 1) << ADDON_MAGNETIC_SPEED;
			else if (bonus->x > ship->x + ADDON_MAGNETIC_DISTANCE)
				bonus->x -= (world->addonLevel[ADDON_MAGNETIC] + 1) << ADDON_MAGNETIC_SPEED;	
		}

//---------------- Draw or destroy bonus -----------------//

		if ((bonus->y += BONUS_SPEED) > ZONE_EDGE_BOTTOM)
		{
			BonusDestroy(world,i);
		}
		else
		{
			GrayClipSprite16_MASK_R((bonus->x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(bonus->y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,16,(unsigned short *)(gfxBonus + bonus->sprite),(unsigned short *)(gfxBonus + bonus->sprite) + 16,(unsigned short *)(gfxBonus + bonus->sprite) + 32,(unsigned short *)(gfxBonus + bonus->sprite) + 32,world->light,world->dark);
			//GrayClipSprite16_MASK_R((bonus->x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(bonus->y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,16,(unsigned short *)(gfxBonus + bonus->sprite),world->light,world->dark);
		}
	}
}
