////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                    Bullet Functions                    //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Fire a new bullet ^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char BulletCreate(struct WORLD *world, unsigned short total, unsigned short type, unsigned short x, unsigned short y, unsigned char shot, unsigned char level, unsigned char way)
{
	unsigned short count;
	struct BULLET *bullet;
	struct WEAPON *weapon;

	if (world->bulletCount + total <= BULLET_COUNT)
	{
		weapon = &weaponList[type];
		count = total;

		while (count--)
		{
			bullet = &world->bulletList[world->bulletCount++];

			bullet->count = count;
			bullet->damage = weapon->damage;
			bullet->effect = weapon->effect;
			bullet->level = level;
			bullet->movement = weapon->movement;
			bullet->radius = weapon->radius << COEF_SCREEN_TO_ZONE;
			bullet->shot = shot;
			bullet->sprite = weapon->sprite;
			bullet->total = total;
			bullet->way = way;
			bullet->x = x + (weapon->xShift << COEF_SCREEN_TO_ZONE);
			bullet->y = y + (way ? (SHIP_SIZE - (weapon->yShift << COEF_SCREEN_TO_ZONE)) : (weapon->yShift << COEF_SCREEN_TO_ZONE));

			bullet->xSpeed = 0;
			bullet->ySpeed = 0;

			(*locationBullet[weapon->location])(world,bullet);
		}

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Destroy a bullet ^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char BulletDestroy(struct WORLD *world, unsigned short index)
{
	if (index < world->bulletCount)
	{
		memcpy(&world->bulletList[index],&world->bulletList[world->bulletCount - 1],sizeof(*world->bulletList));
		world->bulletCount--;

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Update Bullets ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void BulletUpdate(struct WORLD *world)
{
	unsigned short alive, i;
	struct BULLET *bullet;

//-------------- Update all active bullets ---------------//

	for (i = world->bulletCount; i--;)
	{
		bullet = &world->bulletList[i];
		alive = (*movementBullet[bullet->movement])(world,bullet);

//---------------- Draw or destroy bullet ----------------//

		if ((!alive)
		 || (bullet->x < ZONE_EDGE_LEFT - BULLET_SIZE)
		 || (bullet->x > ZONE_EDGE_RIGHT + BULLET_SIZE)
		 || (bullet->y < ZONE_EDGE_TOP - BULLET_SIZE)
		 || (bullet->y > ZONE_EDGE_BOTTOM + BULLET_SIZE))
		{
			BulletDestroy(world,i);
		}
		else
		{
			//GrayClipSprite8_OR_R((bullet->x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(bullet->y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,8,(unsigned char *)(gfxBullet + bullet->sprite),(unsigned char *)(gfxBullet + bullet->sprite) + 8,world->light,world->dark);
			GrayClipSprite8_OR_R((bullet->x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(bullet->y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,8,(unsigned char *)(gfxBullet + bullet->sprite),world->light,world->dark);
		}
	}
}
