////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                    Effect Functions                    //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^ Create a new effect ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char EffectCreate(struct WORLD *world, unsigned short type, unsigned short x, unsigned short y)
{
	struct EFFECT *effect;

	if (world->effectCount < EFFECT_COUNT)
	{
		effect = &world->effectList[world->effectCount++];

		effect->length = effectIndex[type + 1] - effectIndex[type];
		effect->sprite = effectIndex[type];
		effect->time = 0;
		effect->x = x;
		effect->y = y;

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Destroy an effect ^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char EffectDestroy(struct WORLD *world, unsigned short index)
{
	if (index < world->effectCount)
	{
		memcpy(&world->effectList[index],&world->effectList[world->effectCount - 1],sizeof(*world->effectList));
		world->effectCount--;

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Update effects ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void EffectUpdate(struct WORLD *world)
{
	unsigned short alive, i;
	struct EFFECT *effect;

//-------------- Update all active effects ---------------//

	for (i = world->effectCount; i--;)
	{
		effect = &world->effectList[i];
		alive = 1;

//---------------- Next frame or destroy -----------------//

		if (!(effect->time++ & 1))
		{
			if (!(--effect->length))
			{
				EffectDestroy(world,i);
				alive = 0;
			}
			else
			{
				effect->sprite++;
			}
		}

//--------------------- Show effect ----------------------//

		if (alive)
			GrayClipSprite32_TRANW_R((effect->x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(effect->y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,32,(unsigned long *)(gfxEffect + effect->sprite),(unsigned long *)(gfxEffect + effect->sprite) + 32,world->light,world->dark);
	}
}
