////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                    Event Functions                     //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^^ Load level ^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline unsigned char EventInit(struct WORLD *world)
{
	unsigned char *ptr;
	SYM_ENTRY *sym;

	if ((sym = DerefSym(SymFind(SYMSTR(world->level))))
	 && (sym->handle)
	 && (ptr = HeapDeref(sym->handle)))
	{
		world->eventEndArg = 0;
		world->eventEndType = EVENT_END_NONE;

		world->eventId = *(unsigned long *)(ptr + 4);
		world->eventIndex = 0;

		ptr += 8;

		while (*ptr++);
		world->eventPtr = ptr;

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Load savegame ^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline unsigned char EventLoad(struct WORLD *world, const unsigned char *name)
{
	unsigned char *file, *ptr;
	unsigned short i;
	SYM_ENTRY *sym;

	if ((sym = DerefSym(SymFind(SYMSTR(name))))
	 && (sym->handle)
	 && (file = HeapDeref(sym->handle))
	 && (!strcmp(file + *(unsigned short *)file - 3,"VSS"))
	 && (*(file + 2) == EVENT_VERSION))
	{

//---------------- Search for a savegame -----------------//

		i = 4;

		while (i < *(unsigned short *)file - 4)
		{
			if (*(unsigned long *)(file + i + 2) == world->eventId)
			{
				ptr = file + i;
				i = 6;

				memcpy(world,ptr + i,(unsigned long)(&world->eventPtr) - (unsigned long)(&world->eventEndArg));
				i += Even((unsigned long)(&world->eventPtr) - (unsigned long)(&world->eventEndArg));

				memcpy(world->addonLevel,ptr + i,ADDON_COUNT);
				i += Even(ADDON_COUNT);

				memcpy(world->weaponLevel,ptr + i,WEAPON_COUNT);
				i += Even(WEAPON_COUNT);

				memcpy(world->addonList,ptr + i,world->addonCount * sizeof(*world->addonList));
				i += Even(world->addonCount * sizeof(*world->addonList));

				memcpy(world->bonusList,ptr + i,world->bonusCount * sizeof(*world->bonusList));
				i += Even(world->bonusCount * sizeof(*world->bonusList));

				memcpy(world->bulletList,ptr + i,world->bulletCount * sizeof(*world->bulletList));
				i += Even(world->bulletCount * sizeof(*world->bulletList));

				memcpy(world->effectList,ptr + i,world->effectCount * sizeof(*world->effectList));
				i += Even(world->effectCount * sizeof(*world->effectList));

				memcpy(world->messageList,ptr + i,world->messageCount * sizeof(*world->messageList));
				i += Even(world->messageCount * sizeof(*world->messageList));

				memcpy(world->shipList,ptr + i,world->shipCount * sizeof(*world->shipList));
				i += Even(world->shipCount * sizeof(*world->shipList));

				return 1;
			}
			else
			{
				i += *(unsigned short *)(file + i);
			}
		}
	}

//------------------ No savegame found -------------------//

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Proceed events ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline unsigned char EventProceed(struct WORLD *world)
{
	unsigned short cycle, i;

//-------------- Proceed next event or wait --------------//

	do
	{
		cycle = 0;

		switch (world->eventEndType)
		{

//--------------------- Read next event ---------------------//

			case EVENT_END_NONE:
				switch (world->eventPtr[(i = world->eventIndex)])
				{
					case EVENT_END:
						return 0;
					break;

					case EVENT_MESSAGE:
						MessageCreate(world,world->eventPtr + i + 1,0);

						do
						{
							i++;
						} while (world->eventPtr[i]);

						world->eventEndArg = world->eventPtr[i + 2];
						world->eventEndType = world->eventPtr[i + 1];
						world->eventIndex = i + 3;
					break;

					case EVENT_PAUSE:
						world->eventEndArg = world->eventPtr[i + 1];
						world->eventEndType = EVENT_END_PAUSE;
						world->eventIndex += 2;
					break;

					case EVENT_SHOP:
						if (!Shop(world))
							return 2;

						world->eventIndex++;
						cycle = 1;
					break;

					case EVENT_STAR:
						world->eventEndArg = world->eventPtr[i + 3];
						world->eventEndType = world->eventPtr[i + 2];
						world->starAccel = world->eventPtr[i + 1];
						world->eventIndex += 4;
					break;

					case EVENT_WAVE:
						ShipCreate(world,world->eventPtr[i + 1],world->eventPtr[i + 2],world->eventPtr[i + 3],1);

						world->eventEndArg = world->eventPtr[i + 5];
						world->eventEndType = world->eventPtr[i + 4];
						world->eventIndex += 6;
					break;
				}
			break;

//--------------- Pause before next event ----------------//

			case EVENT_END_PAUSE:
				if (!world->eventEndArg--)
				{
					world->eventEndType = EVENT_END_NONE;
					cycle = 1;	
				}
			break;

//------------------ Wait end of event -------------------//

			case EVENT_END_WAIT:
				switch (world->eventEndArg)
				{
					case EVENT_WAIT_MESSAGE:
						if ((!world->messageCount) || (world->messageList[world->messageIndex].state & 4))
						{
							world->eventEndType = EVENT_END_NONE;
							cycle = 1;	
						}
					break;

					case EVENT_WAIT_STAR:
						if (world->starAccel == world->starSpeed)
						{
							world->eventEndType = EVENT_END_NONE;
							cycle = 1;
						}
					break;

					case EVENT_WAIT_WAVE:
						if (world->shipCount == 1)
						{
							world->eventEndType = EVENT_END_NONE;
							cycle = 1;
						}
					break;
				}
			break;
		}
	} while (cycle);

	return 255;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Create savegame ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline unsigned char EventSave(struct WORLD *world, const unsigned char *name)
{
	unsigned char arc = 0, *file = NULL, *ptr;
	unsigned short i, j;
	SYM_ENTRY *sym;

//---------------- Open save or create it ----------------//

	if ((sym = DerefSym(SymFind(SYMSTR(name))))
	 && (sym->handle)
	 && ((file = HeapDeref(sym->handle))))
	{
		if (sym->flags.bits.archived)
		{
			if (!EM_moveSymFromExtMem(SYMSTR(sym->name),HS_NULL))
				file = NULL;

			arc = 1;
		}
	}
	else if ((sym = DerefSym(SymAdd(SYMSTR(name))))
		  && (sym->handle = HeapAlloc(2 + 2 + 6))
		  && (file = HeapDeref(sym->handle)))
	{
		*(unsigned short *)file = 2 + 6;
		*(unsigned char *)(file + 2) = EVENT_VERSION;
		*(unsigned char *)(file + 3) = 0;
	}

//--------- Delete existing saves for this level ---------//

	if (file)
	{
		i = 4;

		while (i < *(unsigned short *)file - 4)
		{
			j = *(unsigned short *)(file + i);

			if (*(unsigned long *)(file + i + 2) == world->eventId)
			{
				memmove(file + i,file + i + j,*(unsigned short *)file - i - j + 1);
				*(unsigned short *)file -= j;
			}
			else
			{
				i += j;
			}
		}

//----------------- Add new save to file -----------------//

		i = 6
		  + Even((unsigned long)(&world->eventPtr) - (unsigned long)(&world->eventEndArg))
		  + Even(ADDON_COUNT)
		  + Even(WEAPON_COUNT)
		  + Even(world->addonCount * sizeof(*world->addonList))
		  + Even(world->bonusCount * sizeof(*world->bonusList))
		  + Even(world->bulletCount * sizeof(*world->bulletList))
		  + Even(world->effectCount * sizeof(*world->effectList))
		  + Even(world->messageCount * sizeof(*world->messageList))
		  + Even(world->shipCount * sizeof(*world->shipList));

		if ((sym->handle = HeapRealloc(sym->handle,*(unsigned short *)file + i + 2))
		 && (file = HeapDeref(sym->handle)))
		{
			ptr = file + *(unsigned short *)file - 4;
			*(unsigned short *)file += i;

			*(unsigned short *)(ptr + 0) = i;
			*(unsigned long *)(ptr + 2) = world->eventId;
			i = 6;

			memcpy(ptr + i,world,(unsigned long)(&world->eventPtr) - (unsigned long)(&world->eventEndArg));
			i += Even((unsigned long)(&world->eventPtr) - (unsigned long)(&world->eventEndArg));

			memcpy(ptr + i,world->addonLevel,ADDON_COUNT);
			i += Even(ADDON_COUNT);

			memcpy(ptr + i,world->weaponLevel,WEAPON_COUNT);
			i += Even(WEAPON_COUNT);

			memcpy(ptr + i,world->addonList,world->addonCount * sizeof(*world->addonList));
			i += Even(world->addonCount * sizeof(*world->addonList));

			memcpy(ptr + i,world->bonusList,world->bonusCount * sizeof(*world->bonusList));
			i += Even(world->bonusCount * sizeof(*world->bonusList));

			memcpy(ptr + i,world->bulletList,world->bulletCount * sizeof(*world->bulletList));
			i += Even(world->bulletCount * sizeof(*world->bulletList));

			memcpy(ptr + i,world->effectList,world->effectCount * sizeof(*world->effectList));
			i += Even(world->effectCount * sizeof(*world->effectList));

			memcpy(ptr + i,world->messageList,world->messageCount * sizeof(*world->messageList));
			i += Even(world->messageCount * sizeof(*world->messageList));

			memcpy(ptr + i,world->shipList,world->shipCount * sizeof(*world->shipList));
			i += Even(world->shipCount * sizeof(*world->shipList));

			memcpy(ptr + i,(unsigned char []){0,'V','S','S',0,OTH_TAG},6);

			if (arc)
				EM_moveSymToExtMem(SYMSTR(sym->name),HS_NULL);

			return 1;
		}

//--------------- Cannot realloc save file ---------------//

		else
		{
			SymDel(SYMSTR(sym->name));
		}
	}

	return 0;
}
