////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//           Factory  0.6 - 11/07/04 - Vertyos           //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^^ Structures ^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//---------------------- File Infos ----------------------//

typedef struct
{
	HANDLE handle;
	HSym hsym;
	unsigned char *data, state;
} FC_FILE;

//-------------------- Keyboard State --------------------//

struct FC_KEYSTATE
{
	unsigned char b1, b2, b3, b4, up, left, down, right, enter, esc;
};



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^^ Prototypes ^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void *FC_Alloc(unsigned short);
void *FC_DataPack(unsigned char *, unsigned short *);
void *FC_DataUnpk(unsigned char *);
void FC_FileClose(FC_FILE *);
FC_FILE *FC_FileMake(const unsigned char *, unsigned short);
FC_FILE *FC_FileOpen(const unsigned char *, unsigned char);
unsigned char *FC_FileOth(FC_FILE *);
void FC_Keyboard(void);
unsigned char FC_Keyboard_Read(unsigned short asm("d0"));
unsigned char FC_Start(unsigned char);
void FC_Stop(void);
void FC_StrDraw(signed short asm("d0"), signed short asm("d1"), const unsigned char * asm("a1"), void * asm("a0"));
void FC_StrDraw0(signed short asm("d0"), signed short asm("d1"), const unsigned char * asm("a1"), void * asm("a0"));
void FC_StrMode(unsigned char asm("d0"));
unsigned short FC_StrWidth0(const unsigned char * asm("a0"));
unsigned short FC_StrWidth1(const unsigned char * asm("a0"));
unsigned short FC_StrWidth2(const unsigned char * asm("a0"));



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^^ Variables ^^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

extern struct FC_KEYSTATE FC_keyState;
extern struct FC_FONTDATA FC_fontData;
extern volatile unsigned long FC_Counter1, FC_Counter5;



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^^^ Defines ^^^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------------ Macros ------------------------//

#define FC_4x6_CharWidth(x) (FC_fontData.size0[x])
#define FC_6x8_CharWidth() 6
#define FC_8x10_CharWidth() 8
#define FC_FileSize(f) (*(unsigned short*)((*(f)).data) + 2)
#define FC_FileTag(f) (*(unsigned char*)((*(f)).data + FC_FileSize(*(f)) - 1))
#define KeyPressed(k) (FC_keyState.k)
#define KeyReleased(k) (!(FC_keyState.k))
#define KeyTest(k...) KeyTestDouble(k)
#define KeyTestDouble(r,c) (FC_Keyboard_Read(~(1 << (r))) & (c))
#define WaitCycle(k) ({ WaitPress(k); WaitRelease(k); })
#define WaitPress(k) ({ do FC_Keyboard(); while (KeyReleased(k)); })
#define WaitRelease(k) ({ do FC_Keyboard(); while (KeyPressed(k)); })

//------------------------ Values ------------------------//

#define FC_4x6 1
#define FC_6x8 2
#define FC_8x10 4
#define FC_All (FC_4x6 | FC_6x8 | FC_8x10)
#define FC_OR 16
#define FC_REP 32
#define FC_XOR 64
#define FC_Archived 1
#define FC_Locked 2
#define FC_Read 0
#define FC_Write 1
#define FC_Redirect 8
#define FC_Gray 16
#define FC_SaveScreen 32
