////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                     Game Function                      //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^^ Game Loop ^^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline unsigned char Game(struct WORLD *world, unsigned char load)
{
	unsigned char event = 0, exit = 255, *keyReleased = world->keyReleased;

//-------------------- Initialisation --------------------//

	WorldInit(world);

	if (!EventInit(world))
	{
		return 3;
	}

	if (load)
	{
		if (!EventLoad(world,"vnssave"))
			return 4;
	}
	else
	{
		ShipCreate(world,1,FLEET_COUNT - 1,MOVEMENT_SHIP_COUNT - 1,0);

		if (!Shop(world))
			return 2;
	}

	StatusCredits(world);
	StatusLife(world);
	StatusScore(world);
	FC_Counter1 = 0;

//---------------------- Main loop -----------------------//

	while (exit == 255)
	{

//----------------- Loop initialisation ------------------//

		FC_Keyboard();
		FC_StrMode(FC_4x6 | FC_OR);

//--------------- Update all active items ----------------//

		event = (world->lost ? 255 : EventProceed(world));
		WorldUpdate(world);
		BulletUpdate(world);
		ShipUpdate(world);
		AddonUpdate(world);
		BonusUpdate(world);
		EffectUpdate(world);
		StatusUpdate(world);
		MessageUpdate(world);

//-------------------- Test keypress ---------------------//

		if (KeyReleased(esc))
		{
			keyReleased[9] = 1;
		}
		else if (keyReleased[9])
		{
			event = 2;

			keyReleased[9] = 0;
		}

//---------------------- Exit game -----------------------//

		if (event == 2)
			exit = 2;
		else if ((world->lost) && (!--world->lost))
			exit = 1;
		else if (event == 0)
			exit = 0;

//--------------------- Draw buffers ---------------------//

#ifdef SWITCH_CPU_USAGE
		unsigned char debug[12];
		sprintf(debug,"CPU : %u%%",(unsigned short)(FC_Counter1 * 100 / IsHW2(13,18)));
		FC_StrDraw0(2 + SCREEN_SHIFT_X,93 - world->messageStep + SCREEN_SHIFT_Y,debug,world->dark);
#endif
		WorldCycle(world);
	}

	return exit;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^ Erase borders on 240*128 screen ^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void EraseBorder(void *buffer)
{
	unsigned short i, j;

	for (i = 0; i < 14; i++)
	{
		for (j = 0; j < 7; j++)
			*(unsigned long *)(buffer + (j << 2)) = 0;

		*(unsigned short *)(buffer + 28) = 0;

		buffer += 30;
	}

	for (i = 14; i < 114; i++)
	{
		*(unsigned long *)(buffer) = 0;
		*(unsigned char *)(buffer + 4) = 0;
		*(unsigned char *)(buffer + 25) = 0;
		*(unsigned long *)(buffer + 26) = 0;

		buffer += 30;
	}

	for (i = 114; i < 128; i++)
	{
		for (j = 0; j < 7; j++)
			*(unsigned long *)(buffer + (j << 2)) = 0;

		*(unsigned short *)(buffer + 28) = 0;

		buffer += 30;
	}
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Random Number ^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned short Random(void)
{
	randSeed = randSeed * 1103515245 + 12345;
	return (unsigned short)((randSeed >> 16) & 32767);
}
