////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                     Menu Functions                     //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Menu Function ^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline unsigned char Menu(struct WORLD *world)
{
	unsigned char autoSelect = 1, buffer[16], esc = 0, exit = 255, game, (*menu)[16], *ptr;
	unsigned short count = 0, i = 0, score, type = 0;
	SYM_ENTRY *sym;

//-------------------- Initialisation --------------------//

	if (Alloc(menu,MENU_ITEM_COUNT * sizeof(*menu)))
	{
		WorldInit(world);

//---------------------- Menu loop -----------------------//

		while (exit == 255)
		{
			do
			{
				switch (type)
				{
					case 0:
						FolderGetCur(menu[0]);

						count = 0;
						sym = SymFindFirst(SYMSTR(menu[0]),FO_SINGLE_FOLDER);

						while ((sym) && (count < MENU_ITEM_COUNT - 1))
						{
							if ((sym->handle)
							 && (ptr = HeapDeref(sym->handle))
							 && (!strcmp(ptr + *(unsigned short *)ptr - 3,"VSL")))
								strcpy(menu[count++],sym->name);

							sym = SymFindNext();
						};

						if (count)
						{
							esc = 0;
							i = 0;
						}
						else
						{
							MessageCreate(world,"\016 No level where found",1);

							autoSelect = 0;
							type = 1;
						}
					break;

					case 1:
						strcpy(menu[0],"New game");
						strcpy(menu[1],"Load game");
						strcpy(menu[2],"Select level");
						strcpy(menu[3],"Credits");
						strcpy(menu[4],"Exit");

						count = 5;
						esc = 1;
					break;
				}
			} while (!count);

//------- Let player select item or autoselect it --------//

			if (autoSelect)
				autoSelect = 0;
			else
				i = MenuSelect(world,menu,count,i,esc);

//--------------------- Test choice ----------------------//

			switch (type)
			{
				case 0:
					if ((sym = SymFindPtr(SYMSTR(menu[i]),0))
					 && (sym->handle)
					 && (ptr = HeapDeref(sym->handle))
					 && (!strcmp(ptr + *(unsigned short *)ptr - 3,"VSL"))
					 && (*(ptr + 2) == EVENT_VERSION))
					{
						MessageCreate(world,ptr + 8,1);
						strcpy(world->level,menu[i]);	
					}
					else
					{
						MessageCreate(world,"\016 Error, unable to open level",1);
					}

					type = 1;
				break;

				case 1:
					switch (i)
					{
						case 0:
						case 1:
							game = Game(world,i);
							score = world->score;
							WorldInit(world);
							MessageCreate(world,messageGame[game],0);

							if (game < 3)
							{
								sprintf(buffer,"Score : %u",score);
								MessageCreate(world,buffer,1);
							}
						break;

						case 2:
							type = 0;
						break;

						case 3:
							MessageCreate(world,"Venus 0.6 - 3rd release - vertyos@fr.st",1);
						break;

						case 4:
							exit = 0;
						break;
					}
				break;
			}
		}

		Free(menu);
	}

	return exit;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^^ Menu Loop ^^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline unsigned char MenuSelect(struct WORLD *world, unsigned char (*menu)[16], unsigned short count, unsigned short choice, unsigned char esc)
{
	signed short roll = 0;
	unsigned char exit = 255, *keyReleased = world->keyReleased;

//-------------------- Initialisation --------------------//

	FC_Counter1 = 0;

//---------------------- Main loop -----------------------//

	while (exit == 255)
	{

//----------------- Loop initialisation ------------------//

		FC_Keyboard();
		FC_StrMode(FC_4x6 | FC_OR);

//-------------------- Test keypress ---------------------//

		if (roll)
		{
			MenuUpdate(world,menu,count,choice,4,roll);

			roll += (roll < 0 ? 4 : - 4);
		}
		else
		{
			MenuUpdate(world,menu,count,choice,3,0);

			if (KeyReleased(up))
			{
				keyReleased[0] = 1;
			}
			else if (keyReleased[0])
			{
				choice = (choice ? choice - 1 : count - 1);
				roll = - MENU_SPACE_Y;

				keyReleased[0] = 0;
			}

			if (KeyReleased(down))
			{
				keyReleased[2] = 1;
			}
			else if (keyReleased[2])
			{
				choice = (choice < count - 1 ? choice + 1 : 0);
				roll = MENU_SPACE_Y;

				keyReleased[2] = 0;	
			}

			if (KeyReleased(b1))
			{
				keyReleased[4] = 1;
			}
			else if (keyReleased[4])
			{
				exit = choice;

				keyReleased[4] = 0;
			}

			if (KeyReleased(esc))
			{
				keyReleased[9] = 1;
			}
			else if (keyReleased[9])
			{
				if (esc)
					exit = count - 1;

				keyReleased[9] = 0;
			}
		}

//--------------- Update all active items ----------------//

		WorldUpdate(world);
		MessageUpdate(world);

//--------------------- Draw buffers ---------------------//

		WorldCycle(world);
	}

	return exit;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^ Menu Update ^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline void MenuUpdate(struct WORLD *world, unsigned char (*menu)[16], unsigned short count, unsigned short choice, unsigned short draw, signed short shift)
{
	unsigned short i;
	signed short x, y = MENU_SHIFT_Y + shift;

//------------------- Draw menu items --------------------//

	choice = (choice ? choice - 1 : count - 1);

	if (shift > 0)
	{
		choice = (choice ? choice - 1 : count - 1);
		y -= MENU_SPACE_Y;
	}
	else if (!shift)
	{
		i = (choice < count - 1 ? choice + 1 : 0);
		x = FC_StrWidth0(menu[i]) >> 1;

		FC_StrDraw0(MENU_SHIFT_X - x - 11,MENU_SHIFT_Y + MENU_SPACE_Y,"[",world->light);
		FC_StrDraw0(MENU_SHIFT_X - x - 10,MENU_SHIFT_Y + MENU_SPACE_Y,"[",world->dark);
		FC_StrDraw0(MENU_SHIFT_X - x - 10,MENU_SHIFT_Y + MENU_SPACE_Y,"[",world->light);
		FC_StrDraw0(MENU_SHIFT_X + x + 9,MENU_SHIFT_Y + MENU_SPACE_Y,"]",world->light);
		FC_StrDraw0(MENU_SHIFT_X + x + 10,MENU_SHIFT_Y + MENU_SPACE_Y,"]",world->dark);
		FC_StrDraw0(MENU_SHIFT_X + x + 10,MENU_SHIFT_Y + MENU_SPACE_Y,"]",world->light);	
	}

	for (i = 0; i < draw; i++)
	{
		x = FC_StrWidth0(menu[choice]) >> 1;

		FC_StrDraw0(MENU_SHIFT_X - x - 1,y,menu[choice],world->light);
		FC_StrDraw0(MENU_SHIFT_X - x,y,menu[choice],world->dark);
		FC_StrDraw0(MENU_SHIFT_X - x,y,menu[choice],world->light);

		y += MENU_SPACE_Y;

		if (++choice == count)
			choice = 0;
	}

	memset(world->dark + (MENU_SHIFT_Y - MENU_ERASE_Y - MENU_SPACE_Y) * 30,0,MENU_SPACE_Y * 30);
	memset(world->dark + (MENU_SHIFT_Y - MENU_ERASE_Y + MENU_SPACE_Y * 3) * 30,0,MENU_SPACE_Y * 30);
	memset(world->light + (MENU_SHIFT_Y - MENU_ERASE_Y - MENU_SPACE_Y) * 30,0,MENU_SPACE_Y * 30);
	memset(world->light + (MENU_SHIFT_Y - MENU_ERASE_Y + MENU_SPACE_Y * 3) * 30,0,MENU_SPACE_Y * 30);

//---------------------- Draw lines ----------------------//

	static const unsigned short line[MENU_LINE_COUNT][2] =
	{
		{MENU_SHIFT_X - 55,MENU_SHIFT_X - 50},
		{MENU_SHIFT_X - 45,MENU_SHIFT_X - 35},
		{MENU_SHIFT_X - 30,MENU_SHIFT_X + 29},
		{MENU_SHIFT_X + 34,MENU_SHIFT_X + 44},
		{MENU_SHIFT_X + 49,MENU_SHIFT_X + 54},
	};

	for (i = MENU_LINE_COUNT; i--;)
	{
		LineH_OR(line[i][0],line[i][1],MENU_SHIFT_Y - MENU_ERASE_Y - 1,world->dark);
		LineH_OR(line[i][0],line[i][1],MENU_SHIFT_Y - MENU_ERASE_Y + MENU_SPACE_Y * 3,world->dark);
	}

//--------------------- Draw logo ---------------------//

	GrayClipSprite32_TRANW_R(32 + SCREEN_SHIFT_X,MENU_SHIFT_LOGO,32,(unsigned long *)(gfxTitle + 0),(unsigned long *)(gfxTitle + 0) + 32,world->light,world->dark);
	GrayClipSprite32_TRANW_R(64 + SCREEN_SHIFT_X,MENU_SHIFT_LOGO,32,(unsigned long *)(gfxTitle + 1),(unsigned long *)(gfxTitle + 1) + 32,world->light,world->dark);
	GrayClipSprite32_TRANW_R(96 + SCREEN_SHIFT_X,MENU_SHIFT_LOGO,32,(unsigned long *)(gfxTitle + 2),(unsigned long *)(gfxTitle + 2) + 32,world->light,world->dark);
	
}
