////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                   Message Functions                    //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Queue message ^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MessageCreate(struct WORLD *world, const unsigned char *str, unsigned char breakable)
{
	struct MESSAGE *message;

	if (world->messageCount < MESSAGE_COUNT)
	{
		message = &world->messageList[(world->messageIndex + world->messageCount++) % MESSAGE_COUNT];

		message->end = 0;
		message->pause = 0;
		message->start = 0;
		message->state = breakable ? 4 : 0;
		message->str = str;

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Update messages ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void MessageUpdate(struct WORLD *world)
{
	unsigned char str[MESSAGE_MAXSIZE];
	unsigned short i, j;
	struct MESSAGE *message;

//---------------------- Draw panel ----------------------//

	if ((i = world->messageStep))
	{
		memset(world->dark + (MESSAGE_SHIFT_Y + 6 - i) * 30,255,30);
		memset(world->light + (MESSAGE_SHIFT_Y + 6 - i) * 30,0,i * 30);
		memset(world->dark + (MESSAGE_SHIFT_Y + 7 - i) * 30,0,(i - 1) * 30);	
	}

//------------------- Messages queued --------------------//

	if (world->messageCount)
	{

//-------------------- Activate panel --------------------//
		if (world->messageStep < 8)
		{
			world->messageStep++;
		}

//----------------- Show current message -----------------//
		else
		{
			message = &world->messageList[world->messageIndex];
			j = ((message->end - message->start < MESSAGE_MAXSIZE - 1) ? message->end - message->start : MESSAGE_MAXSIZE - 1);

			memcpy(str,message->str + message->start,j);
			str[j] = 0;

			FC_StrDraw0(2 + SCREEN_SHIFT_X,MESSAGE_SHIFT_Y,str,world->dark);

			if ((message->state & 4) && (world->messageCount > 1))
			{
				if (++world->messageIndex == MESSAGE_COUNT)
					world->messageIndex = 0;

				world->messageCount--;
			}
			else if (message->pause)
			{
				message->pause--;
			}
			else if (message->state & 2)
			{
				if (++world->messageIndex == MESSAGE_COUNT)
					world->messageIndex = 0;

				world->messageCount--;
			}
			else
			{
				if (message->state & 1)
				{
					message->start = message->end;
					message->state &= ~ 1;
				}

				switch (message->str[message->end++])
				{
					case 0:
						message->pause = 32;
						message->state |= 2;
					break;

					case ' ':
						j = message->end;

						while ((message->str[j]) && (message->str[j] != ' '))
							j++;

						j = ((j - message->start < MESSAGE_MAXSIZE - 1) ? j - message->start : MESSAGE_MAXSIZE - 1);

						memcpy(str,message->str + message->start,j);
						str[j] = 0;

						if (FC_StrWidth0(str) > MESSAGE_MAXLENGTH)
						{
							message->pause = 32;
							message->state |= 1;
						}
					break;
				}
			}
		}
	}

//------------------ Desactivate panel -------------------//

	else if (world->messageStep)
	{
		world->messageStep--;
	}
}
