////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                Bullets & Ships movements               //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^ Bullet default move ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveBulletStraight(struct WORLD *world, struct BULLET *bullet)
{
	bullet->x += bullet->xSpeed;
	bullet->y += bullet->ySpeed;

	return 1;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^ Bullet shift left or right ^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveBulletShift(struct WORLD *world, struct BULLET *bullet)
{
	bullet->x += ((Random() & 1) ? 1 : - 1)  << COEF_SCREEN_TO_ZONE;
	bullet->y += bullet->ySpeed;

	return 1;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^ Sprite fade animation ^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveBulletFade(struct WORLD *world, struct BULLET *bullet)
{
	if ((!(world->time & 3)) && (++bullet->sprite == 10))
		return 0;

	bullet->x += bullet->xSpeed;
	bullet->y += bullet->ySpeed;

	return 1;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Bullet reverse way ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveBulletReverse(struct WORLD *world, struct BULLET *bullet)
{
	if (bullet->way)
	{
		if (bullet->ySpeed < 64)
			bullet->ySpeed += 4;
	}
	else
	{
		if (bullet->ySpeed > - 64)
			bullet->ySpeed -= 4;
	}

	if ((!(world->time & 1)) && (++bullet->sprite == 17))
		bullet->sprite = 13;

	bullet->x += bullet->xSpeed;
	bullet->y += bullet->ySpeed;

	return 1;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^ Bullet follow ennemy ^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveBulletFollow(struct WORLD *world, struct BULLET *bullet)
{
	if ((bullet->x < world->shipList[bullet->count].x)
	 && (bullet->xSpeed < 32))
		bullet->xSpeed++;
	else if ((bullet->x > world->shipList[bullet->count].x)
	 && (bullet->xSpeed > - 32))
		bullet->xSpeed--;

	bullet->x += bullet->xSpeed;
	bullet->y += bullet->ySpeed;

	return 1;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^ Bullet horizontal move ^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveBulletLateral(struct WORLD *world, struct BULLET *bullet)
{
	unsigned short i = 0;

	if ((!bullet->xSpeed)
	 || ((i = bullet->count) >= world->shipCount)
	 || ((bullet->xSpeed < 0) == (bullet->x < world->shipList[i].x)))
	{
		bullet->xSpeed = 0;
		bullet->ySpeed = (bullet->way ? 32 : - 32);
	}

	bullet->x += bullet->xSpeed;
	bullet->y += bullet->ySpeed;

	return 1;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Player movements ^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipHuman(struct WORLD *world, struct SHIP *ship)
{
	unsigned char fire;
	unsigned short i;

	signed short pointList[2][4] =
	{
		 {0,16,ship->xStart,ship->yStart + 1024},
		 {0,8 ,ship->xStart,ship->yStart + 1280}
	};

//------------------ Keyboard controls -------------------//

	if (ship->step == 2)
	{
		fire = ShipMoveHuman(world,ship);
	}

//-------------------- Auto controls ---------------------//

	else
	{
		ShipMoveAuto(world,ship,pointList,2,2);

		if (ship->step == 2)
		{
			ship->xSpeed = 0;
			ship->ySpeed = 0;
		}

		fire = 0;
	}

//------------------------ Flamme ------------------------//

	if (world->starSpeed > 31)
	{
		i = (world->time & 14) >> 1;

		GrayClipSprite16_MASK_R(ship->x >> COEF_SCREEN_TO_ZONE,(ship->y + 224) >> COEF_SCREEN_TO_ZONE,16,(unsigned short *)(gfxFlamme + i),(unsigned short *)(gfxFlamme + i) + 16,(unsigned short *)(gfxFlamme + i) + 32,(unsigned short *)(gfxFlamme + i) + 32,world->light,world->dark);	
		//GrayClipSprite16_MASK_R(ship->x >> COEF_SCREEN_TO_ZONE,(ship->y + 224) >> COEF_SCREEN_TO_ZONE,16,(unsigned short *)(gfxFlamme + i),world->light,world->dark);	
	}

//----------------- Return firing state ------------------//

	return fire;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Losange movement ^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipLosangeLeft(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[5][4] =
	{
		 {8,8,ship->xStart,      ship->yStart      },
		 {8,4,ship->xStart,      ship->yStart      },
		 {8,4,ship->xStart - 256,ship->yStart + 128},
		 {8,4,ship->xStart,      ship->yStart + 256},
		 {8,4,ship->xStart + 256,ship->yStart + 128}
	};

	return ShipMoveAuto(world,ship,pointList,1,5);
}


////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Losange movement ^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipLosangeRight(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[5][4] =
	{
		 {8,8,ship->xStart,      ship->yStart      },
		 {8,4,ship->xStart,      ship->yStart      },
		 {8,4,ship->xStart + 256,ship->yStart + 128},
		 {8,4,ship->xStart,      ship->yStart + 256},
		 {8,4,ship->xStart - 256,ship->yStart + 128}
	};

	return ShipMoveAuto(world,ship,pointList,1,5);
}


////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^ Left patrol movement ^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipPatrolLeft(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[11][4] =
	{
		 {16,16,ship->xStart - 1024,ship->yStart + 256},
		 {8 ,8, ship->xStart - 1024,ship->yStart + 256},
		 {8, 8, ship->xStart - 768, ship->yStart + 512},
		 {16,16,ship->xStart - 256, ship->yStart      },
		 {16,16,ship->xStart + 256, ship->yStart + 512},
		 {8, 8, ship->xStart + 768, ship->yStart      },
		 {8, 8, ship->xStart + 1024,ship->yStart + 256},
		 {8, 8, ship->xStart + 768, ship->yStart + 512},
		 {16,16,ship->xStart + 256, ship->yStart      },
		 {16,16,ship->xStart - 256, ship->yStart + 512},
		 {8, 8, ship->xStart - 768, ship->yStart      }
	};

	return ShipMoveAuto(world,ship,pointList,1,11);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^ Right patrol movement ^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipPatrolRight(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[11][4] =
	{
		 {16,16,ship->xStart + 1024,ship->yStart + 256},
		 {8 ,8, ship->xStart + 1024,ship->yStart + 256},
		 {8, 8, ship->xStart + 768, ship->yStart + 512},
		 {16,16,ship->xStart + 256, ship->yStart      },
		 {16,16,ship->xStart - 256, ship->yStart + 512},
		 {8, 8, ship->xStart - 768, ship->yStart      },
		 {8, 8, ship->xStart - 1024,ship->yStart + 256},
		 {8, 8, ship->xStart - 768, ship->yStart + 512},
		 {16,16,ship->xStart - 256, ship->yStart      },
		 {16,16,ship->xStart + 256, ship->yStart + 512},
		 {8, 8, ship->xStart + 768, ship->yStart      }
	};

	return ShipMoveAuto(world,ship,pointList,1,11);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^ Left rotation movement ^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipRotateLeft(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[7][4] =
	{
		{16,32,ship->xStart - 1024,ship->yStart + 256},
		{16,16,ship->xStart - 1024,ship->yStart + 256},
		{16,16,ship->xStart - 768, ship->yStart + 512},
		{32,16,ship->xStart + 768, ship->yStart + 512},
		{16,16,ship->xStart + 1024,ship->yStart + 256},
		{16,16,ship->xStart + 768, ship->yStart      },
		{32,16,ship->xStart - 768, ship->yStart      }
	};

	return ShipMoveAuto(world,ship,pointList,1,7);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^ Right rotation movement ^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipRotateRight(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[7][4] =
	{
		{16,32,ship->xStart + 1024,ship->yStart + 256},
		{16,16,ship->xStart + 1024,ship->yStart + 256},
		{16,16,ship->xStart + 768, ship->yStart + 512},
		{32,16,ship->xStart - 768, ship->yStart + 512},
		{16,16,ship->xStart - 1024,ship->yStart + 256},
		{16,16,ship->xStart - 768, ship->yStart      },
		{32,16,ship->xStart + 768, ship->yStart      }
	};

	return ShipMoveAuto(world,ship,pointList,1,7);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Square movement ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipSquareLeft(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[5][4] =
	{
		 {8,16,ship->xStart,      ship->yStart      },
		 {8,8, ship->xStart - 256,ship->yStart      },
		 {8,8, ship->xStart - 256,ship->yStart + 256},
		 {8,8, ship->xStart + 256,ship->yStart + 256},
		 {8,8, ship->xStart + 256,ship->yStart      }
	};

	return ShipMoveAuto(world,ship,pointList,1,5);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Square movement ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipSquareRight(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[5][4] =
	{
		 {8,16,ship->xStart,      ship->yStart      },
		 {8,8, ship->xStart + 256,ship->yStart      },
		 {8,8, ship->xStart + 256,ship->yStart + 256},
		 {8,8, ship->xStart - 256,ship->yStart + 256},
		 {8,8, ship->xStart - 256,ship->yStart      }
	};

	return ShipMoveAuto(world,ship,pointList,1,5);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^ Left eight movement ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipEightLeft(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[7][4] =
	{
		{16,32,ship->xStart - 1024,ship->yStart + 256},
		{16,16,ship->xStart - 1024,ship->yStart + 256},
		{16,16,ship->xStart - 768, ship->yStart + 512},
		{32,16,ship->xStart + 768, ship->yStart      },
		{16,16,ship->xStart + 1024,ship->yStart + 256},
		{16,16,ship->xStart + 768, ship->yStart + 512},
		{32,16,ship->xStart - 768, ship->yStart      }
	};

	return ShipMoveAuto(world,ship,pointList,1,7);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^ Right eight movement ^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipEightRight(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[7][4] =
	{
		{16,32,ship->xStart + 1024,ship->yStart + 256},
		{16,16,ship->xStart + 1024,ship->yStart + 256},
		{16,16,ship->xStart + 768, ship->yStart + 512},
		{32,16,ship->xStart - 768, ship->yStart      },
		{16,16,ship->xStart - 1024,ship->yStart + 256},
		{16,16,ship->xStart - 768, ship->yStart + 512},
		{32,16,ship->xStart + 768, ship->yStart      }
	};

	return ShipMoveAuto(world,ship,pointList,1,7);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^ Left cross movement ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipCrossLeft(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[5][4] =
	{
		{16,16,ship->xStart - 768,ship->yStart      },
		{32,16,ship->xStart - 768,ship->yStart      },
		{16,4, ship->xStart + 768,ship->yStart + 512},
		{32,16,ship->xStart - 768,ship->yStart + 512},
		{16,4, ship->xStart + 768,ship->yStart      }
	};

	return ShipMoveAuto(world,ship,pointList,1,5);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^ Right cross movement ^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char MoveShipCrossRight(struct WORLD *world, struct SHIP *ship)
{
	signed short pointList[5][4] =
	{
		{16,16,ship->xStart + 768,ship->yStart      },
		{32,16,ship->xStart + 768,ship->yStart      },
		{16,4, ship->xStart - 768,ship->yStart + 512},
		{32,16,ship->xStart - 768,ship->yStart + 512},
		{16,4, ship->xStart - 768,ship->yStart      }
	};

	return ShipMoveAuto(world,ship,pointList,1,5);
}
