////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                     Ship Functions                     //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^ Wave Initialisation ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char ShipCreate(struct WORLD *world, unsigned short total, unsigned short type, unsigned short movement, unsigned char way)
{
	unsigned short count;
	struct FLEET *fleet;
	struct SHIP *ship;

//----------------- Create "count" ships -----------------//

	if (world->shipCount + total <= SHIP_COUNT)
	{
		fleet = &fleetList[type];
		count = total;

		while (count--)
		{
			ship = &world->shipList[world->shipCount++];

			ship->count = count;
			ship->effect = fleet->effect;
			ship->life = fleet->life;
			ship->movement = movement;
			ship->radius = fleet->radius << COEF_SCREEN_TO_ZONE;
			ship->sprite = fleet->sprite;
			ship->total = total;
			ship->way = way;
			ship->weapon = fleet->weapon;

			ship->fire = 0;
			ship->hit = 0;
			ship->pos = 0;
			ship->reload = 0;
			ship->shot = 0;
			ship->speedAdd = 2;
			ship->speedMax = 32;
			ship->step = 0;
			ship->xSpeed = 0;
			ship->ySpeed = 0;

			(*locationShip[movementLocation[movement]])(world,ship);
		}

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Destroy a ship ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char ShipDestroy(struct WORLD *world, unsigned short index)
{
	if (index < world->shipCount)
	{
		memcpy(&world->shipList[index],&world->shipList[world->shipCount - 1],sizeof(*world->shipList));
		world->shipCount--;

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^ Move a ship, following waypoints ^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char ShipMoveAuto(struct WORLD *world, struct SHIP *ship, signed short (*pointList)[4], unsigned short pointLoop, unsigned short pointTotal)
{
	signed short new, xAim, xAlt, xSpeed, yAim, yAlt, ySpeed;
	unsigned short next;

//------ Register new waypoint when current reached ------//

	if (!(ship->pos & ((1 << SHIP_MOVE_DONE_X) | (1 << SHIP_MOVE_DONE_Y))))
	{
		if (ship->pos & (1 << SHIP_MOVE_NEXT))
		{
			xAlt = pointList[ship->step][2];
			yAlt = pointList[ship->step][3];

			if (++ship->step == pointTotal)
				ship->step = pointLoop;
		}
		else
		{
			xAlt = ship->x;
			yAlt = ship->y;

			ship->step = 0;
		}

		xAim = pointList[ship->step][2];
		yAim = pointList[ship->step][3];

		ship->pos = ((xAlt != xAim) << SHIP_MOVE_DONE_X)
		          | ((yAlt != yAim) << SHIP_MOVE_DONE_Y)
		          | ((xAlt > xAim) << SHIP_MOVE_STATE_X)
		          | ((yAlt > yAim) << SHIP_MOVE_STATE_Y)
		          | (1 << SHIP_MOVE_NEXT);
	}
	else
	{
		xAim = pointList[ship->step][2];
		yAim = pointList[ship->step][3];
	}

//-------- Define next waypoint and maximum speed --------//

	next = ((ship->step + 1 == pointTotal) ? pointLoop : ship->step + 1);
	xAlt = pointList[next][2];
	yAlt = pointList[next][3];
	xSpeed = pointList[ship->step][0];
	ySpeed = pointList[ship->step][1];

//------------- Move to waypoint X position --------------//

	if ((ship->pos & (1 << SHIP_MOVE_DONE_X))
	 && (!!(ship->pos & (1 << SHIP_MOVE_STATE_X)) == (ship->x > xAim)))
	{
		new = ship->x + ship->xSpeed * (ship->xSpeed / ship->speedAdd + 1);

		if (ship->x > xAim)
		{
			if ((ship->xSpeed < 0) && (new < xAim) && (new < xAlt))
			{
				ship->xSpeed += ship->speedAdd;
			}
			else
			{
				if (ship->xSpeed < - xSpeed)
					ship->xSpeed += ship->speedAdd;
				else if ((ship->xSpeed -= ship->speedAdd) < - xSpeed)
					ship->xSpeed = - xSpeed;
			}
		}
		else
		{
			if ((ship->xSpeed > 0) && (new > xAim) && (new > xAlt))
			{
				ship->xSpeed -= ship->speedAdd;
			}
			else
			{
				if (ship->xSpeed > xSpeed)
					ship->xSpeed -= ship->speedAdd;
				else if ((ship->xSpeed += ship->speedAdd) > xSpeed)
					ship->xSpeed = xSpeed;
			}
		}
	}
	else
	{
		if ((xAim == xAlt) || ((ship->xSpeed > 0) == (ship->x > xAlt)))
			ship->xSpeed = 0;

		ship->pos &= ~ (1 << SHIP_MOVE_DONE_X);
	}

//------------- Move to waypoint Y position --------------//

	if ((ship->pos & (1 << SHIP_MOVE_DONE_Y))
	 && (!!(ship->pos & (1 << SHIP_MOVE_STATE_Y)) == (ship->y > yAim)))
	{
		new = ship->y + ship->ySpeed * (ship->ySpeed / ship->speedAdd + 1);

		if (ship->y > yAim)
		{
			if ((ship->ySpeed < 0) && (new < yAim) && (new < yAlt))
			{
				ship->ySpeed += ship->speedAdd;
			}
			else
			{
				if (ship->ySpeed < - ySpeed)
					ship->ySpeed += ship->speedAdd;
				else if ((ship->ySpeed -= ship->speedAdd) < - ySpeed)
					ship->ySpeed = - ySpeed;
			}
		}
		else
		{
			if ((ship->ySpeed > 0) && (new > yAim) && (new > yAlt))
			{
				ship->ySpeed -= ship->speedAdd;
			}
			else
			{
				if (ship->ySpeed > ySpeed)
					ship->ySpeed -= ship->speedAdd;
				else if ((ship->ySpeed += ship->speedAdd) > ySpeed)
					ship->ySpeed = ySpeed;
			}
		}
	}
	else
	{
		if ((yAim == yAlt) || ((ship->ySpeed > 0) == (ship->y > yAlt)))
			ship->ySpeed = 0;

		ship->pos &= ~ (1 << SHIP_MOVE_DONE_Y);
	}

//------------------ Change ship coords ------------------//

	ship->x += ship->xSpeed;
	ship->y += ship->ySpeed;

//---------------------- Use weapon ----------------------//

	if (ship->fire)
	{
		ship->fire--;
	}
	else
	{
		switch (ship->weapon >> 2)
		{
			case 0:
				ship->fire = ((Random() & 1023) < 5);
			break;

			case 1:
				ship->fire = ((Random() & 1023) < 4);
			break;

			case 2:
				ship->fire = (((Random() & 1023) < 3) ? (Random() & 63) : 0);
			break;

			case 3:
				ship->fire = (((Random() & 1023) < 2) ? (Random() & 31) : 0);
			break;

			case 4:
				ship->fire = (((Random() & 1023) < 2) ? (Random() & 31) : 0);
			break;

			case 5:
				ship->fire = ((Random() & 1023) < 3);
			break;

			case 6:
				ship->fire = ((Random() & 1023) < 4);
			break;

			case 7:
				ship->fire = ((Random() & 1023) < 3);
			break;

			case 8:
				ship->fire = ((Random() & 1023) < 4);
			break;

			case 9:
				ship->fire = ((Random() & 1023) < 3);
			break;
		}
	}

//----------------- Return firing state ------------------//

	return (!ship->reload) && (ship->fire);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^ Move a ship, controled by keyboard ^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char ShipMoveHuman(struct WORLD *world, struct SHIP *ship)
{
	unsigned char fire, *keyReleased = world->keyReleased;
	unsigned short i;
	unsigned long *keyDelay = world->keyDelay, time = world->time;

//---------------- Move ship left / right ----------------//

	if (KeyPressed(left))
	{
		if ((ship->xSpeed -= (ship->speedAdd << 1)) < - ship->speedMax)
		{
			ship->xSpeed = - ship->speedMax;
		}

		if (keyReleased[3])
		{
			if ((keyDelay[3] > time - KEY_DELAY) && (world->addonLevel[ADDON_TELEPORT] != 255))
			{
				EffectCreate(world,5,ship->x + (SHIP_SIZE >> 1) - (EFFECT_SIZE >> 1),ship->y + (SHIP_SIZE >> 1) - (EFFECT_SIZE >> 1));
				
				ship->x -= (world->addonLevel[ADDON_TELEPORT] + 1) << ADDON_TELEPORT_DISTANCE;
			}

			keyDelay[3] = time;
			keyReleased[3] = 0;
		}
	}
	else
	{
		keyReleased[3] = 1;
	}

	if (KeyPressed(right))
	{
		if ((ship->xSpeed += (ship->speedAdd << 1)) > ship->speedMax)
		{
			ship->xSpeed = ship->speedMax;
		}

		if (keyReleased[1])
		{
			if ((keyDelay[1] > time - KEY_DELAY) && (world->addonLevel[ADDON_TELEPORT] != 255))
			{
				EffectCreate(world,5,ship->x + (SHIP_SIZE >> 1) - (EFFECT_SIZE >> 1),ship->y + (SHIP_SIZE >> 1) - (EFFECT_SIZE >> 1));
				ship->x += (world->addonLevel[ADDON_TELEPORT] + 1) << ADDON_TELEPORT_DISTANCE;
			}

			keyDelay[1] = time;
			keyReleased[1] = 0;
		}
	}
	else
	{
		keyReleased[1] = 1;
	}

	if (keyReleased[1] && keyReleased[3])
	{
		if (ship->xSpeed > 0)
		{
			if ((ship->xSpeed -= ship->speedAdd) < 0)
				ship->xSpeed = 0;
		}
		else if (ship->xSpeed < 0)
		{
			if ((ship->xSpeed += ship->speedAdd) > 0)
				ship->xSpeed = 0;
		}
	}

//----------------- Move ship up / down ------------------//

	if (KeyPressed(up))
	{
		if ((ship->ySpeed -= (ship->speedAdd << 1)) < - ship->speedMax)
			ship->ySpeed = - ship->speedMax;

		keyReleased[0] = 0;
	}
	else
	{
		keyReleased[0] = 1;
	}

	if (KeyPressed(down))
	{
		if ((ship->ySpeed += (ship->speedAdd << 1)) > ship->speedMax)
			ship->ySpeed = ship->speedMax;

		keyReleased[2] = 0;
	}
	else
	{
		keyReleased[2] = 1;
	}

	if (keyReleased[0] && keyReleased[2])
	{
		if (ship->ySpeed > 0)
		{
			if ((ship->ySpeed -= ship->speedAdd) < 0)
				ship->ySpeed = 0;
		}
		else if (ship->ySpeed < 0)
		{
			if ((ship->ySpeed += ship->speedAdd) > 0)
				ship->ySpeed = 0;
		}
	}

	ship->x += ship->xSpeed;
	ship->y += ship->ySpeed;

	if (ship->x < ZONE_EDGE_LEFT)
	{
		ship->x = 0;
		ship->xSpeed = 0;
	}
	else if (ship->x > ZONE_EDGE_RIGHT - SHIP_SIZE + 1)
	{
		ship->x = ZONE_EDGE_RIGHT - SHIP_SIZE + 1;
		ship->xSpeed = 0;
	}

	if (ship->y < ZONE_EDGE_TOP)
	{
		ship->y = 0;
		ship->ySpeed = 0;
	}
	else if (ship->y > ZONE_EDGE_BOTTOM - SHIP_SIZE + 1)
	{
		ship->y = ZONE_EDGE_BOTTOM - SHIP_SIZE + 1;
		ship->ySpeed = 0;
	}

//------------------------- Fire -------------------------//

	if (KeyPressed(b1))
	{
		keyReleased[4] = 0;
		fire = 1;
	}
	else
	{
		keyReleased[4] = 1;
		fire = 0;
	}

//-------------------- Change weapon ---------------------//

	if (KeyReleased(b2))
	{
		keyReleased[5] = 1;
	}
	else if (keyReleased[5])
	{
		i = ship->weapon >> 2;

		do
		{
			if (++i == WEAPON_COUNT)
				i = 0;
		} while (world->weaponLevel[i] == 255);

		ship->weapon = (i << 2) + world->weaponLevel[i];
		keyReleased[5] = 0;

		MessageCreate(world,weaponName[i],1);
	}

//---------------- Show status indicators ----------------//

	if (KeyReleased(b3))
	{
		keyReleased[6] = 1;
	}
	else if (keyReleased[6])
	{
		StatusCredits(world);
		StatusLife(world);
		StatusScore(world);
	}

//----------------- Return firing state ------------------//

	return fire;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^ Update Ships ^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void ShipUpdate(struct WORLD *world)
{
	unsigned char alive;
	unsigned short i, j;
	struct BONUS *bonus;
	struct BULLET *bullet;
	struct SHIP *ship;

//--------------- Update all active ships ----------------//

	for (i = world->shipCount; i--;)
	{
		ship = &world->shipList[i];
		alive = 1;

//------------------------ Shoot -------------------------//

		if ((*movementShip[ship->movement])(world,ship) && (!ship->reload))
		{
			if ((!i) && (!world->lost) && (world->score))
				world->score--;

			BulletCreate(world,weaponList[ship->weapon].count,ship->weapon,ship->x + (SHIP_SIZE >> 1) - (BULLET_SIZE >> 1),ship->y - (BULLET_SIZE >> 1),ship->shot++,(ship->weapon & 3),ship->way);

			ship->reload = weaponList[ship->weapon].reload;
		}

//------------------------ Reload ------------------------//

		else if (ship->reload)
		{
			ship->reload--;
		}

//------------------- Bonus collisions -------------------//

		if ((!i) && (!world->lost))
		{
			for (j = world->bonusCount; j--;)
			{
				bonus = &world->bonusList[j];
	
				if (Collide(BONUS_RADIUS,bonus->x,bonus->y,ship->radius,ship->x,ship->y))
				{
					if ((world->credits += bonus->value) > 990)
						world->credits = 990;

					StatusCredits(world);
					BonusDestroy(world,j);
				}
			}
		}

//------------------ Bullet collisions -------------------//

		for (j = world->bulletCount; j--;)
		{
			bullet = &world->bulletList[j];

			if ((bullet->way != ship->way)
			 && (Collide(bullet->radius,bullet->x,bullet->y,ship->radius,ship->x,ship->y)))
			{
				if ((!i) && (!world->lost))
				{
					if ((world->addonLevel[ADDON_DEFLECTOR] != 255)
					 && ((Random() & 1023) < deflectorProb[world->addonLevel[ADDON_DEFLECTOR]]))
					{
						AddonCreate(world,ADDON_DEFLECTOR);

						bullet->ySpeed = - bullet->ySpeed;
						bullet->way = 1 - bullet->way;
						alive = 1;
					}
					else
					{
						StatusLife(world);

						alive = 0;
					}
				}
				else
				{
					alive = 0;
				}

				if (!alive)
				{
					if (bullet->effect != 255)
						EffectCreate(world,bullet->effect,bullet->x + (BULLET_SIZE >> 1) - (EFFECT_SIZE >> 1),bullet->y + (BULLET_SIZE >> 1) - (EFFECT_SIZE >> 1));

					ship->hit = SHIP_HIT;
					alive = ((ship->life -= bullet->damage) > 0);

					BulletDestroy(world,j);
				}
			}
		}

//---------------------- Draw ship -----------------------//

		if (alive)
		{
			if (ship->way)
			{
				GrayClipSprite16_MASK_R((ship->x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(ship->y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,16,(unsigned short *)(gfxEnnemy + ship->sprite),(unsigned short *)(gfxEnnemy + ship->sprite) + 16,(unsigned short *)(gfxEnnemy + ship->sprite) + 32,(unsigned short *)(gfxEnnemy + ship->sprite) + 32,world->light,(ship->hit ? world->light : world->dark));
				//GrayClipSprite16_MASK_R((ship->x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(ship->y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,16,(unsigned short *)(gfxEnnemy + ship->sprite),world->light,(ship->hit ? world->light : world->dark));
			}
			else
			{
				if (!(world->time & 1))
				{
					j = weaponSprite[ship->weapon >> 2];

					if (!ship->sprite)
						ship->sprite = (j ? (j << 1) - 1 : 0);
					else if (ship->sprite == (j << 1) - 1)
						ship->sprite++;
					else if (ship->sprite != (j << 1))
						ship->sprite = ((ship->sprite & 1) ? 0 : ship->sprite - 1);
				}

				GrayClipSprite16_MASK_R((ship->x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(ship->y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,16,(unsigned short *)(gfxPlayer + ship->sprite),(unsigned short *)(gfxPlayer + ship->sprite) + 16,(unsigned short *)(gfxPlayer + ship->sprite) + 32,(unsigned short *)(gfxPlayer + ship->sprite) + 32,world->light,(ship->hit ? world->light : world->dark));
				//GrayClipSprite16_MASK_R((ship->x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(ship->y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,16,(unsigned short *)(gfxPlayer + ship->sprite),world->light,(ship->hit ? world->light : world->dark));
			}

			if (ship->hit)
				ship->hit--;
		}

//--------------------- Destroy ship ---------------------//

		else
		{
			if (ship->effect != 255)
				EffectCreate(world,ship->effect,ship->x + (SHIP_SIZE >> 1) - (EFFECT_SIZE >> 1),ship->y + (SHIP_SIZE >> 1) - (EFFECT_SIZE >> 1));

			if ((!i) && (!world->lost))
			{
				world->lost = 32;
			}
			else
			{
				j = (Random() & 1023);

				if (j < BONUS_PROB_HIGH)
					BonusCreate(world,2,ship->x,ship->y);
				else if (j < BONUS_PROB_MED)
					BonusCreate(world,1,ship->x,ship->y);
				else if (j < BONUS_PROB_LOW)
					BonusCreate(world,0,ship->x,ship->y);

				world->score += (ship->weapon + 1) << 2;
			}

			ShipDestroy(world,i);
		}
	}
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Select weapon ^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void ShipWeapon(struct WORLD *world)
{
	unsigned short i;
	register struct SHIP *ship = &world->shipList[0];

	i = ship->weapon >> 2;

	while (world->weaponLevel[i] == 255)
		i++;

	ship->weapon = (i << 2) + world->weaponLevel[i];
}
