////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                     Shop Functions                     //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Shop Function ^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

unsigned char Shop(struct WORLD *world)
{
	unsigned char exit = 255, *keyReleased = world->keyReleased, str[MESSAGE_MAXSIZE];
	unsigned short choice = 0, cur = 0, i, update = 1;

//-------------------- Initialisation --------------------//

	world->fade = 0;
	FC_Counter1 = 0;

//---------------------- Main loop -----------------------//

	while (exit == 255)
	{

//----------------- Loop initialisation ------------------//

		FC_Keyboard();

//-------------------- Test keypress ---------------------//

		if (KeyReleased(enter))
		{
			keyReleased[8] = 1;
		}
		else if (keyReleased[8])
		{
			for (i = 0; i < WEAPON_COUNT; i++)
			{
				if (world->weaponLevel[i] != 255)
					break;
			}

			if (i == WEAPON_COUNT)
			{
				strcpy(str,"You need a weapon to start !");
				cur = 0;
			}
			else
			{
				exit = 1;
			}
		
			keyReleased[8] = 0;
		}

		if (KeyReleased(esc))
		{
			keyReleased[9] = 1;
		}
		else if (keyReleased[9])
		{
			exit = 0;

			keyReleased[9] = 0;
		}

//--------------------- Update shop ----------------------//

		switch ((i = ShopSelect(world,&choice)))
		{
			case 0:
				cur = 0;
				update = 1;
			break;

			case 1 ... 8:
				strcpy(str,messageShop[i - 1]);
				cur = 0;
			break;
		}

		ShopDraw(world,str,choice,&cur,update);
		update = 0;

//--------------------- Draw buffers ---------------------//

		WorldCycle(world);
	}

//------------------- Re-select weapon -------------------//

	if (exit)
		ShipWeapon(world);

	world->fade = 0;

	return exit;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^^ Draw Shop ^^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline void ShopDraw(struct WORLD *world, unsigned char *str, unsigned short choice, unsigned short *cur, unsigned char update)
{
	unsigned char buffer[MESSAGE_MAXSIZE], stop;
	unsigned short i, j;

//--------------------- Draw header ----------------------//

	memset(world->dark + SHOP_HEADER_SHIFT * 30,255,30);
	memset(world->light + SHOP_HEADER_SHIFT * 30,255,30);
	memset(world->dark + (SHOP_HEADER_SHIFT + 20) * 30,255,30);
	memset(world->light + (SHOP_HEADER_SHIFT + 20) * 30,255,30);

	FC_StrMode(FC_4x6 | FC_OR);
	FC_StrDraw0(2 + SCREEN_SHIFT_X,SHOP_HEADER_SHIFT + 4,":: Shop ::",world->dark);
	FC_StrDraw0(2 + SCREEN_SHIFT_X,SHOP_HEADER_SHIFT + 4,":: Shop ::",world->light);
	sprintf(buffer,"Credits : %u $",world->credits);
	FC_StrDraw0(42 + SCREEN_SHIFT_X,SHOP_HEADER_SHIFT + 4,buffer,world->dark);
	FC_StrDraw0(42 + SCREEN_SHIFT_X,SHOP_HEADER_SHIFT + 4,buffer,world->light);
	sprintf(buffer,"Life : %d%%",world->shipList[0].life);
	FC_StrDraw0(102 + SCREEN_SHIFT_X,SHOP_HEADER_SHIFT + 4,buffer,world->dark);
	FC_StrDraw0(102 + SCREEN_SHIFT_X,SHOP_HEADER_SHIFT + 4,buffer,world->light);
	FC_StrDraw0(2 + SCREEN_SHIFT_X,SHOP_HEADER_SHIFT + 12,":: [2nd] Buy - [Enter] Continue - [\177] Save",world->dark);
	FC_StrDraw0(2 + SCREEN_SHIFT_X,SHOP_HEADER_SHIFT + 12,":: [2nd] Buy - [Enter] Continue - [\177] Save",world->light);

//---------------------- Draw items ----------------------//

	memset(world->dark + (SCREEN_EDGE_TOP + SCREEN_SHIFT_Y) * 30,255,30);
	memset(world->light + (SCREEN_EDGE_TOP + SCREEN_SHIFT_Y) * 30,255,30);
	memset(world->dark + (SCREEN_EDGE_BOTTOM + SCREEN_SHIFT_Y) * 30,255,30);
	memset(world->light + (SCREEN_EDGE_BOTTOM + SCREEN_SHIFT_Y) * 30,255,30);

	LineV_OR(SCREEN_EDGE_LEFT + SCREEN_SHIFT_X,SCREEN_EDGE_TOP + SCREEN_SHIFT_Y,SCREEN_EDGE_BOTTOM + SCREEN_SHIFT_Y,world->dark);
	LineV_OR(SCREEN_EDGE_LEFT + SCREEN_SHIFT_X,SCREEN_EDGE_TOP + SCREEN_SHIFT_Y,SCREEN_EDGE_BOTTOM + SCREEN_SHIFT_Y,world->light);
	LineV_OR(SCREEN_EDGE_RIGHT + SCREEN_SHIFT_X,SCREEN_EDGE_TOP + SCREEN_SHIFT_Y,SCREEN_EDGE_BOTTOM + SCREEN_SHIFT_Y,world->dark);
	LineV_OR(SCREEN_EDGE_RIGHT + SCREEN_SHIFT_X,SCREEN_EDGE_TOP + SCREEN_SHIFT_Y,SCREEN_EDGE_BOTTOM + SCREEN_SHIFT_Y,world->light);
	LineV_OR(78 + SCREEN_SHIFT_X,SHOP_ITEMS_SHIFT,8 * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT,world->dark);
	LineV_OR(78 + SCREEN_SHIFT_X,SHOP_ITEMS_SHIFT,8 * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT,world->light);
	LineV_OR(81 + SCREEN_SHIFT_X,SHOP_ITEMS_SHIFT,8 * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT,world->dark);
	LineV_OR(81 + SCREEN_SHIFT_X,SHOP_ITEMS_SHIFT,8 * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT,world->light);

	for (i = 0; i < 8; i++)
	{
		j = i * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT;

		memset(world->dark + (i * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT) * 30,255,30);
		memset(world->light + (i * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT) * 30,255,30);

		FC_StrDraw0(2 + SCREEN_SHIFT_X,i * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT + 2,weaponName[i],world->dark);
		FC_StrDraw0(2 + SCREEN_SHIFT_X,i * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT + 2,weaponName[i],world->light);
		ShopIndic(world,2 + 64 + SCREEN_SHIFT_X,j,world->weaponLevel[i]);

		if (i < 2)
		{
			FC_StrDraw0(83 + SCREEN_SHIFT_X,j + 2,weaponName[i + 8],world->dark);
			FC_StrDraw0(83 + SCREEN_SHIFT_X,j + 2,weaponName[i + 8],world->light);
			ShopIndic(world,83 + 64 + SCREEN_SHIFT_X,j,world->weaponLevel[i + 8]);
		}
		else if ((i > 2) && (i < 6))
		{
			FC_StrDraw0(83 + SCREEN_SHIFT_X,j + 2,addonName[i - 3],world->dark);
			FC_StrDraw0(83 + SCREEN_SHIFT_X,j + 2,addonName[i - 3],world->light);
			ShopIndic(world,83 + 64 + SCREEN_SHIFT_X,j,world->addonLevel[i - 3]);
		}
		else if (i == 7)
		{
			FC_StrDraw0(83 + SCREEN_SHIFT_X,j + 2,"50% life repair",world->dark);
			FC_StrDraw0(83 + SCREEN_SHIFT_X,j + 2,"50% life repair",world->light);
		}
	}

	memset(world->dark + (8 * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT) * 30,255,30);
	memset(world->light + (8 * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT) * 30,255,30);

//------------------- Draw description -------------------//

	if (update)
	{
		switch (choice)
		{
			case 0 ... 9:
				i = (world->weaponLevel[choice] + 1) & 255;
	
				if (i == 4)
					sprintf(str,"%s is fully upgraded",weaponName[choice]);
				else
					sprintf(str,"%s level %u : %u $",weaponName[choice],i + 1,weaponCost[(choice << 2) + i]);
			break;
	
			case 11 ... 13:
				i = (world->addonLevel[choice - 11] + 1) & 255;
	
				if (i == 4)
					sprintf(str,"%s is fully upgraded",addonName[choice - 11]);
				else
					sprintf(str,"%s level %u : %u $",addonName[choice - 11],i + 1,addonCost[((choice - 11) << 2) + i]);
			break;
	
			case 15:
				if (world->shipList[0].life == 100)
					strcpy(str,"Your ship isn't damaged");
				else
					strcpy(str,"50% life repair : 10 $");
			break;
	
			default:
				strcpy(str,"---");
			break;
		}
	}

	if (*cur < strlen(str))
		(*cur)++;

	stop = str[*cur];
	str[*cur] = 0;

	memset(world->dark + SHOP_DESCRIPTION_SHIFT * 30,255,30);
	memset(world->light + SHOP_DESCRIPTION_SHIFT * 30,255,30);

	FC_StrDraw0(2 + SCREEN_SHIFT_X,SHOP_DESCRIPTION_SHIFT + 2,str,world->dark);
	FC_StrDraw0(2 + SCREEN_SHIFT_X,SHOP_DESCRIPTION_SHIFT + 2,str,world->light);

	str[*cur] = stop;

//------------------- Highlight choice -------------------//

	j = (choice & 7) * SHOP_ITEMS_SPACE + SHOP_ITEMS_SHIFT + 1;

	if (choice < 8)
	{
		for (i = j; i < j + 7; i++)
			LineH_XOR(1 + SCREEN_SHIFT_X,77 + SCREEN_SHIFT_X,i,world->dark);
	}
	else
	{
		for (i = j; i < j + 7; i++)
			LineH_XOR(82 + SCREEN_SHIFT_X,158 + SCREEN_SHIFT_X,i,world->dark);
	}
}


////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Level Indicator ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline void ShopIndic(struct WORLD *world, unsigned short x, unsigned short y, unsigned char level)
{
	unsigned short i, high = ((level + 1) & 255);

	for (i = 0; i < 4; i++)
	{
		LineV_OR(x,y + 2,y + 6,(i < high ? world->dark : world->light));
		x += 2;
	}
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^ Shop Select ^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline unsigned char ShopSelect(struct WORLD *world, unsigned short *choice)
{
	unsigned char *keyReleased = world->keyReleased;
	unsigned short i;

//--------------------- Move cursor ----------------------//

	if (KeyReleased(up))
	{
		keyReleased[0] = 1;
	}
	else if (keyReleased[0])
	{
		*choice = ((*choice & 7) ? *choice - 1 : *choice + 7);

		keyReleased[0] = 0;

		return 0;
	}

	if (KeyReleased(right))
	{
		keyReleased[1] = 1;
	}
	else if (keyReleased[1])
	{
		*choice ^= 8;

		keyReleased[1] = 0;

		return 0;
	}

	if (KeyReleased(down))
	{
		keyReleased[2] = 1;
	}
	else if (keyReleased[2])
	{
		*choice = ((*choice & 7) == 7 ? *choice - 7 : *choice + 1);

		keyReleased[2] = 0;

		return 0;
	}

	if (KeyReleased(left))
	{
		keyReleased[3] = 1;
	}
	else if (keyReleased[3])
	{
		*choice ^= 8;

		keyReleased[3] = 0;

		return 0;
	}

//----------------------- Buy item -----------------------//

	if (KeyReleased(b1))
	{
		keyReleased[4] = 1;
	}
	else if (keyReleased[4])
	{
		keyReleased[4] = 0;

		switch (*choice)
		{			
			case 0 ... 9:
				i = (world->weaponLevel[*choice] + 1) & 255;

				if (i == 4)
				{
					return 1;
				}
				else if (world->credits < weaponCost[(*choice << 2) + i])
				{
					return 3;
				}
				else
				{
					world->weaponLevel[*choice] = i;
					world->credits -= weaponCost[(*choice << 2) + i];

					return 0;
				}
			break;

			case 11 ... 13:
				i = (world->addonLevel[*choice - 11] + 1) & 255;

				if (i == 4)
				{
					return 1;
				}
				else if (world->credits < addonCost[((*choice - 11) << 2) + i])
				{
					return 3;
				}
				else
				{
					world->addonLevel[*choice - 11] = i;
					world->credits -= addonCost[((*choice - 11) << 2) + i];

					return 0;
				}
			break;

			case 15:
				if (world->shipList[0].life == 100)
				{
					return 2;
				}
				else if (world->credits < 10)
				{
					return 3;
				}
				else
				{
					if ((world->shipList[0].life += 50) > 100)
						world->shipList[0].life = 100;

					world->credits -= 10;

					return 0;
				}
			break;

			default:
				return 4;
			break;
		}
	}

//---------------------- Save game -----------------------//

	if (KeyReleased(b3))
	{
		keyReleased[6] = 1;
	}
	else if (keyReleased[6])
	{
		keyReleased[6] = 0;

		return (world->eventIndex ? (EventSave(world,"vnssave") ? 5 : 6) : 7);
	}

	return 255;
}
