////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                    Status Functions                    //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Show credits bar ^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline void StatusCredits(struct WORLD *world)
{
	StatusShow(&world->statusCredits);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Show life bar ^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline void StatusLife(struct WORLD *world)
{
	StatusShow(&world->statusLife);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ Show score bar ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline void StatusScore(struct WORLD *world)
{
	StatusShow(&world->statusScore);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Show status bar ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void StatusShow(unsigned short *status)
{
	if (!(*status))
		*status = STATUS_TIME_ANIM + STATUS_TIME_STAY;
	else if ((*status <= STATUS_TIME_STAY) && (*status > STATUS_TIME_ANIM))
		*status =  STATUS_TIME_STAY;
	else if (*status <= STATUS_TIME_ANIM)
		*status = STATUS_TIME_ANIM + STATUS_TIME_STAY - *status;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^ Draw status bars ^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void StatusUpdate(struct WORLD *world)
{
	signed char life = (world->lost ? 0 : world->shipList[0].life);
	unsigned char credits[6];
	unsigned short d;

//--------------------- Credits bar ----------------------//

	if (world->statusCredits)
	{
		if (world->statusCredits < STATUS_TIME_ANIM)
			d = STATUS_TIME_ANIM - world->statusCredits;
		else if (world->statusCredits < STATUS_TIME_STAY)
			d = 0;
		else
			d = world->statusCredits - STATUS_TIME_STAY;

		GrayClipSprite32_MASK_R(SCREEN_STATUS_CREDITS_X + SCREEN_SHIFT_X,SCREEN_STATUS_CREDITS_Y - d + SCREEN_SHIFT_Y,9,(unsigned long *)(gfxStatus),(unsigned long *)(gfxStatus) + 9,(unsigned long *)(gfxStatus) + 18,(unsigned long *)(gfxStatus) + 18,world->light,world->dark);

		sprintf(credits,"%u $",world->credits);
		FC_StrMode(FC_4x6 | FC_OR);
		FC_StrDraw(SCREEN_STATUS_CREDITS_X + 5 + SCREEN_SHIFT_X,SCREEN_STATUS_CREDITS_Y - d + 2 + SCREEN_SHIFT_Y,credits,world->dark);

		world->statusCredits--;
	}

//---------------------- Score bar -----------------------//

	if (world->statusScore)
	{
		if (world->statusScore < STATUS_TIME_ANIM)
			d = STATUS_TIME_ANIM - world->statusScore;
		else if (world->statusScore < STATUS_TIME_STAY)
			d = 0;
		else
			d = world->statusScore - STATUS_TIME_STAY;

		GrayClipSprite32_MASK_R(SCREEN_STATUS_SCORE_X + SCREEN_SHIFT_X,SCREEN_STATUS_SCORE_Y - d + SCREEN_SHIFT_Y,9,(unsigned long *)(gfxStatus + 1),(unsigned long *)(gfxStatus + 1) + 9,(unsigned long *)(gfxStatus + 1) + 18,(unsigned long *)(gfxStatus + 1) + 18,world->light,world->dark);

		sprintf(credits,"%u",world->score);
		FC_StrMode(FC_4x6 | FC_OR);
		FC_StrDraw(SCREEN_STATUS_SCORE_X + 6 + SCREEN_SHIFT_X,SCREEN_STATUS_SCORE_Y - d + 2 + SCREEN_SHIFT_Y,credits,world->dark);

		world->statusScore--;
	}

//----------------------- Life bar -----------------------//

	if (world->statusLife)
	{
		if (world->statusLife < STATUS_TIME_ANIM)
			d = STATUS_TIME_ANIM - world->statusLife;
		else if (world->statusLife < STATUS_TIME_STAY)
			d = 0;
		else
			d = world->statusLife - STATUS_TIME_STAY;

		GrayClipSprite32_MASK_R(SCREEN_STATUS_LIFE_X + SCREEN_SHIFT_X,SCREEN_STATUS_LIFE_Y - d + SCREEN_SHIFT_Y,9,(unsigned long *)(gfxStatus + 2),(unsigned long *)(gfxStatus + 2) + 9,(unsigned long *)(gfxStatus + 2) + 18,(unsigned long *)(gfxStatus + 2) + 18,world->light,world->dark);

		if (life)
		{
			life = ((life - 1) >> 2);

			LineH_OR(SCREEN_STATUS_LIFE_X + 4 + SCREEN_SHIFT_X,SCREEN_STATUS_LIFE_X + 4 + life + SCREEN_SHIFT_X,SCREEN_STATUS_LIFE_Y - d + 3 + SCREEN_SHIFT_Y,world->dark);
			LineH_OR(SCREEN_STATUS_LIFE_X + 3 + SCREEN_SHIFT_X,SCREEN_STATUS_LIFE_X + 4 + life + SCREEN_SHIFT_X,SCREEN_STATUS_LIFE_Y - d + 4 + SCREEN_SHIFT_Y,world->light);
			LineH_OR(SCREEN_STATUS_LIFE_X + 3 + SCREEN_SHIFT_X,SCREEN_STATUS_LIFE_X + 3 + life + SCREEN_SHIFT_X,SCREEN_STATUS_LIFE_Y - d + 5 + SCREEN_SHIFT_Y,world->dark);
		}

		world->statusLife--;
	}
}
