////////////////////////////////////////////////////////////
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//                    World Functions                     //
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ Defines & Includes ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

//------------------- Header Includes --------------------//

#include "TIGCChdr.h"
#include "Factory.h"
#include "Constants.h"
#include "Aliases.h"
#include "Structures.h"
#include "Prototypes.h"
#include "Globals.h"



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^ World Creation ^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline unsigned char WorldCreate(struct WORLD *world)
{
	unsigned short i;

#ifdef SWITCH_DBUFFERING
	#define BUFFER_SIZE (GRAYDBUFFER_SIZE)
#else
	#define BUFFER_SIZE (LCD_SIZE << 1)
#endif

	if (Alloc(world->buffer,Even(BUFFER_SIZE)
	                      + Even(ADDON_COUNT * sizeof(*world->addonList))
	                      + Even(BONUS_COUNT * sizeof(*world->bonusList))
	                      + Even(BULLET_COUNT * sizeof(*world->bulletList))
	                      + Even(EFFECT_COUNT * sizeof(*world->effectList))
	                      + Even(MESSAGE_COUNT * sizeof(*world->messageList))
	                      + Even(SHIP_COUNT * sizeof(*world->shipList))
	                      + Even(STAR1_COUNT * sizeof(*world->star1))
	                      + Even(STAR2_COUNT * sizeof(*world->star2))))
	{
		i = Even(BUFFER_SIZE);
		world->addonList = world->buffer + i;

		i += Even(ADDON_COUNT * sizeof(*world->addonList));
		world->bonusList = world->buffer + i;

		i += Even(BONUS_COUNT * sizeof(*world->bonusList));
		world->bulletList = world->buffer + i;

		i += Even(BULLET_COUNT * sizeof(*world->bulletList));
		world->effectList = world->buffer + i;

		i += Even(EFFECT_COUNT * sizeof(*world->effectList));
		world->messageList = world->buffer + i;

		i += Even(MESSAGE_COUNT * sizeof(*world->messageList));
		world->shipList = world->buffer + i;

		i += Even(SHIP_COUNT * sizeof(*world->shipList));
		world->star1 = world->buffer + i;

		i += Even(STAR1_COUNT * sizeof(*world->star1));
		world->star2 = world->buffer + i;

#ifdef SWITCH_DBUFFERING
		GrayDBufInit(world->buffer);

		world->dark = GrayDBufGetHiddenPlane(DARK_PLANE);
		world->light = GrayDBufGetHiddenPlane(LIGHT_PLANE);
#else
		world->dark = world->buffer + LCD_SIZE;
		world->light = world->buffer;
#endif

		memset(world->keyDelay,0,8 * sizeof(*world->keyDelay));
		memset(world->keyReleased,1,8 * sizeof(*world->keyReleased));
		memset(world->level,0,9 * sizeof(*world->level));
		memset(world->dark,0,LCD_SIZE);
		memset(world->light,0,LCD_SIZE);

		for (i = STAR1_COUNT; i--;)
		{
			world->star1[i][0] = random(ZONE_SIZE_HORZ - STAR1_SIZE);
			world->star1[i][1] = random(ZONE_SIZE_VERT);
		}

		for (i = STAR2_COUNT; i--;)
		{
			world->star2[i][0] = random(ZONE_SIZE_HORZ - STAR2_SIZE);
			world->star2[i][1] = random(ZONE_SIZE_VERT);
		}

		return 1;
	}

	return 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ World Destruction ^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

inline void WorldFree(struct WORLD *world)
{
	Free(world->buffer);
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^ World End of Cycle ^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void WorldCycle(struct WORLD *world)
{
#ifndef SWITCH_TI89
	EraseBorder(world->dark);
	EraseBorder(world->light);

	LineH_OR(40,199,13,world->light);
	LineH_OR(40,199,114,world->light);
	LineV_OR(39,14,113,world->light);
	LineV_OR(200,14,113,world->light);
#endif

#ifdef SWITCH_DBUFFERING
	GrayDBufToggleSync();
	GrayDBufSetActiveIdxSync(world->time & 1);

	world->dark = GrayDBufGetHiddenPlane(DARK_PLANE);
	world->light = GrayDBufGetHiddenPlane(LIGHT_PLANE);
#else
	unsigned short i = world->fade * 30;

	memcpy(GrayGetPlane(0) + Is89(50,64) * 30 - i,world->light + Is89(50,64) * 30 - i,i << 1);
	memcpy(GrayGetPlane(1) + Is89(50,64) * 30 - i,world->dark + Is89(50,64) * 30 - i,i << 1);

	if (world->fade < Is89(50,64))
	{
		if ((world->fade += SCREEN_FADE) > Is89(50,64))
			world->fade = Is89(50,64);

		memset(GrayGetPlane(0) + Is89(50,64) * 30 - i,255,30);
		memset(GrayGetPlane(1) + Is89(50,64) * 30 - i,255,30);
		memset(GrayGetPlane(0) + Is89(50,64) * 30 + i,255,30);
		memset(GrayGetPlane(1) + Is89(50,64) * 30 + i,255,30);
	}
#endif

	world->time++;

	memset(world->dark,0,LCD_SIZE);
	memset(world->light,0,LCD_SIZE);

	while (FC_Counter1 < IsHW2(13,18));
	FC_Counter1 = 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^ World Initialisation ^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void WorldInit(struct WORLD *world)
{
	memset(world->addonLevel,255,ADDON_COUNT * sizeof(*world->addonLevel));
	memset(world->weaponLevel,255,WEAPON_COUNT * sizeof(*world->weaponLevel));

	world->addonCount = 0;
	world->bonusCount = 0;
	world->bulletCount = 0;
	world->credits = 20;
	world->effectCount = 0;
	world->eventEndArg = 0;
	world->eventEndType = 0;
	world->eventIndex = 0;
	world->eventPtr = NULL;
	world->fade = 0;
	world->lost = 0;
	world->messageCount = 0;
	world->messageIndex = 0;
	world->messageStep = 0;
	world->score = 0;
	world->shipCount = 0;
	world->starAccel = 8;
	world->starSpeed = 8;
	world->statusCredits = 0;
	world->statusLife = 0;
	world->statusScore = 0;
	world->time = 0;
}



////////////////////////////////////////////////////////////
//^^^^^^^^^^^^^^^^^^^^^^ Draw stars ^^^^^^^^^^^^^^^^^^^^^^//
////////////////////////////////////////////////////////////

void WorldUpdate(struct WORLD *world)
{
	signed short *x, *y;
	unsigned short i, speed1, speed2;

//---------------------- Set speed -----------------------//

	if (world->starAccel < world->starSpeed)
		world->starSpeed--;
	else if (world->starAccel > world->starSpeed)
		world->starSpeed++;

	speed1 = (world->starSpeed << 1);
	speed2 = world->starSpeed;

//---------------------- Big Stars -----------------------//

	for (i = STAR1_COUNT; i--;)
	{
		x = &world->star1[i][0];
		y = &world->star1[i][1];

		if ((*y += speed1) > ZONE_EDGE_BOTTOM)
		{
			*x = random(ZONE_SIZE_HORZ - STAR1_SIZE);
			*y = ZONE_EDGE_TOP - STAR1_SIZE - random(STAR1_SIZE);
		}

		//GrayClipSprite8_OR_R((*x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(*y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,8,(unsigned char *)(gfxStar),(unsigned char *)(gfxStar) + 8,world->light,world->dark);
		GrayClipSprite8_OR_R((*x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(*y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,8,(unsigned char *)(gfxStar),world->light,world->dark);
	}

//--------------------- Small Stars ----------------------//

	for (i = STAR2_COUNT; i--;)
	{
		x = &world->star2[i][0];
		y = &world->star2[i][1];

		if ((*y += speed2) > ZONE_EDGE_BOTTOM)
		{
			*x = random(ZONE_SIZE_HORZ - STAR2_SIZE);
			*y = ZONE_EDGE_TOP - STAR2_SIZE - random(STAR2_SIZE);
		}

		//GrayClipSprite8_OR_R((*x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(*y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,8,(unsigned char *)(gfxStar + 1),(unsigned char *)(gfxStar + 1) + 8,world->light,world->dark);
		GrayClipSprite8_OR_R((*x >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_X,(*y >> COEF_SCREEN_TO_ZONE) + SCREEN_SHIFT_Y,8,(unsigned char *)(gfxStar + 1),world->light,world->dark);
	}
}
